@@ -30,41 +30,13 @@
 #endif
 
 #ifdef ENABLE_BROWSER_INTERFACE
-#include <strings.h>
-#ifndef USE_MAEMO
-#include <libgnome/gnome-url.h>
-#else
-#include <tablet-browser-interface.h>
-#endif
-#endif
-
-#ifdef ENABLE_BROWSER_INTERFACE
-#ifndef USE_MAEMO
-static void browser_url(char *url) {
-  /* taken from gnome-open, part of libgnome */
-  GError *err = NULL;
-  gnome_url_show(url, &err);
-}
-#else
-static void browser_url(osso_context_t *osso_context, char *url) {
-  osso_rpc_run_with_defaults(osso_context, "osso_browser",
-			     OSSO_BROWSER_OPEN_NEW_WINDOW_REQ, NULL,
-			     DBUS_TYPE_STRING, url,
-			     DBUS_TYPE_BOOLEAN, FALSE, DBUS_TYPE_INVALID);
-}
-#endif
-
 static gboolean on_link_clicked(GtkWidget *widget, GdkEventButton *event,
 				gpointer user_data) {
 
   const char *str = 
     gtk_label_get_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget))));
   
-  browser_url(
-#ifdef USE_MAEMO
-	      (osso_context_t *)user_data,
-#endif
-	      (char*)str);  
+  browser_url(GTK_WIDGET(user_data), (char*)str);  
 
   return TRUE;
 }
@@ -83,13 +55,7 @@ static GtkWidget *link_new(GtkWidget *parent, const char *url) {
     gtk_container_add(GTK_CONTAINER(eventbox), label);
     
     g_signal_connect(eventbox, "button-press-event", 
-		     G_CALLBACK(on_link_clicked), 
-#ifndef USE_MAEMO
-		     NULL
-#else
-		     g_object_get_data(G_OBJECT(parent), "osso-context")
-#endif
-		     );
+		     G_CALLBACK(on_link_clicked), parent);
 
     return eventbox;
   }
@@ -103,16 +69,13 @@ static GtkWidget *link_new(GtkWidget *parent, const char *url) {
 
 #ifdef ENABLE_BROWSER_INTERFACE
 void on_paypal_button_clicked(GtkButton *button, gpointer user_data) {
-  browser_url(
-#ifdef USE_MAEMO
-	      (osso_context_t *)user_data,
-#endif
+  browser_url(GTK_WIDGET(user_data),
 	      "https://www.paypal.com/cgi-bin/webscr?"
 	      "cmd=_s-xclick&hosted_button_id=7400558");
 }
 #endif
 
-GtkWidget *label_big(char *str) {
+static GtkWidget *label_big(char *str) {
   GtkWidget *label = gtk_label_new("");
   char *markup = 
     g_markup_printf_escaped("<span size='x-large'>%s</span>", str);
@@ -121,7 +84,7 @@ GtkWidget *label_big(char *str) {
   return label;
 }
 
-GtkWidget *label_xbig(char *str) {
+static GtkWidget *label_xbig(char *str) {
   GtkWidget *label = gtk_label_new("");
   char *markup = 
     g_markup_printf_escaped("<span size='xx-large'>%s</span>", str);
@@ -136,7 +99,7 @@ on_label_realize(GtkWidget *widget, gpointer user_data)  {
   gtk_widget_set_size_request(widget, widget->parent->allocation.width, -1);
 }
 
-GtkWidget *label_wrap(char *str) {
+static GtkWidget *label_wrap(char *str) {
   GtkWidget *label = gtk_label_new(str);
 
   gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD);
@@ -202,9 +165,9 @@ GtkWidget *copyright_page_new(GtkWidget *parent) {
   GtkWidget *ihbox = gtk_hbox_new(FALSE, 20);
   gtk_box_pack_start(GTK_BOX(ihbox), 
 #ifdef MAEMO5
-		     icon_get(ihbox, "maep.64"),
+		     icon_get_widget(ihbox, "maep.64"),
 #else
-		     icon_get(ihbox, "maep.32"),
+		     icon_get_widget(ihbox, "maep.32"),
 #endif
 		     FALSE, FALSE, 0);
 
@@ -277,14 +240,14 @@ GtkWidget *authors_page_new(GtkWidget *parent) {
 
   /* -------------------------------------------- */
   ivbox = gtk_vbox_new(FALSE, 0);
-  author_add(ivbox, _("Original map widget by:"));
+  author_add(ivbox, _("Original osm-gps-map widget by:"));
   author_add(ivbox, "John Stowers <john.stowers@gmail.com>");
   author_add(ivbox, "Marcus Bauer <marcus.bauer@gmail.com>"),
   gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
 
   /* -------------------------------------------- */
   ivbox = gtk_vbox_new(FALSE, 0);
-  author_add(ivbox, _("Search feature provided by:"));
+  author_add(ivbox, _("Search and wikipedia feature provided by:"));
   link_add(ivbox, parent, _("http://geonames.org"));
   gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
 
@@ -325,20 +288,14 @@ GtkWidget *donate_page_new(GtkWidget *parent) {
   GtkWidget *button = gtk_button_new();
   gtk_button_set_image(GTK_BUTTON(button), 
 #ifdef MAEMO5
-		     icon_get(ihbox, "paypal.64")
+		     icon_get_widget(ihbox, "paypal.64")
 #else
-		     icon_get(ihbox, "paypal.32")
+		     icon_get_widget(ihbox, "paypal.32")
 #endif
 		       );
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   g_signal_connect(button, "clicked", 
-		   G_CALLBACK(on_paypal_button_clicked), 
-#ifndef USE_MAEMO
-		   NULL
-#else
-		   g_object_get_data(G_OBJECT(parent), "osso-context")
-#endif
-		   );
+		   G_CALLBACK(on_paypal_button_clicked), parent);
  
   gtk_box_pack_start(GTK_BOX(ihbox), button, TRUE, FALSE, 0);
   gtk_box_pack_start_defaults(GTK_BOX(vbox), ihbox);
