@@ -4,6 +4,7 @@
  * osm-gps-map.c
  * Copyright (C) Marcus Bauer 2008 <marcus.bauer@gmail.com>
  * Copyright (C) John Stowers 2009 <john.stowers@gmail.com>
+ * Copyright (C) Till Harbaum 2009 <till@harbaum.org>
  *
  * Contributions by
  * Everaldo Canuto 2009 <everaldo.canuto@gmail.com>
@@ -36,36 +37,18 @@
 #include <glib/gstdio.h>
 #include <glib/gprintf.h>
 #include <libsoup/soup.h>
+#include <cairo.h>
 
 #include "converter.h"
 #include "osm-gps-map-types.h"
 #include "osm-gps-map.h"
 
-#ifdef USE_CAIRO
-#include <cairo.h>
-#endif
-
 #define ENABLE_DEBUG 0
 
 #define EXTRA_BORDER (TILESIZE / 2)
 
 #define OSM_GPS_MAP_SCROLL_STEP 10
 
-/* any defined key enables key support */
-#if (defined(OSM_GPS_MAP_KEY_FULLSCREEN) || \
-     defined(OSM_GPS_MAP_KEY_ZOOMIN) || \
-     defined(OSM_GPS_MAP_KEY_ZOOMOUT) || \
-     defined(OSM_GPS_MAP_KEY_UP) || \
-     defined(OSM_GPS_MAP_KEY_DOWN) || \
-     defined(OSM_GPS_MAP_KEY_LEFT) || \
-     defined(OSM_GPS_MAP_KEY_RIGHT))
-#define OSM_GPS_MAP_KEYS
-#endif
-
-#ifdef OSM_GPS_MAP_KEYS
-#include <gdk/gdkkeysyms.h>
-#endif
-
 #define USER_AGENT "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.11) Gecko/20071127 Firefox/2.0.0.11"
 
 struct _OsmGpsMapPrivate
@@ -98,6 +81,7 @@ struct _OsmGpsMapPrivate
 
     //where downloaded tiles are cached
     char *tile_dir;
+    char *tile_base_dir;
     char *cache_dir;
 
     //contains flags indicating the various special characters
@@ -124,12 +108,7 @@ struct _OsmGpsMapPrivate
 #ifdef OSD_DOUBLE_BUFFER
     GdkPixmap *dbuf_pixmap;
 #endif
-#endif
-
-#ifdef OSM_GPS_MAP_KEY_FULLSCREEN
-    gboolean fullscreen;
-#endif
-    
+#endif    
     //additional images or tracks added to the map
     GSList *tracks;
     GSList *images;
@@ -156,6 +135,11 @@ struct _OsmGpsMapPrivate
     int ui_gps_point_inner_radius;
     int ui_gps_point_outer_radius;
 
+    //For storing keybindings
+    guint keybindings[OSM_GPS_MAP_KEY_MAX];
+
+    guint fullscreen : 1;
+    guint keybindings_enabled : 1;
     guint is_disposed : 1;
     guint dragging : 1;
     guint button_down : 1;
@@ -182,6 +166,8 @@ enum
     PROP_REPO_URI,
     PROP_PROXY_URI,
     PROP_TILE_CACHE_DIR,
+    PROP_TILE_CACHE_BASE_DIR,
+    PROP_TILE_CACHE_DIR_IS_FULL_PATH,
     PROP_ZOOM,
     PROP_MAX_ZOOM,
     PROP_MIN_ZOOM,
@@ -552,7 +538,7 @@ osm_gps_map_print_images (OsmGpsMap *map)
                          priv->gc_map,
                          im->image,
                          0,0,
-                         x-(im->w/2),y-(im->h/2),
+                         x-im->xoffset,y-im->yoffset,
                          im->w,im->h,
                          GDK_RGB_DITHER_NONE, 0, 0);
 
@@ -586,15 +572,9 @@ osm_gps_map_draw_gps_point (OsmGpsMap *map)
         map_y0 = priv->map_y - EXTRA_BORDER;
         x = lon2pixel(priv->map_zoom, priv->gps->rlon) - map_x0;
         y = lat2pixel(priv->map_zoom, priv->gps->rlat) - map_y0;
-#ifdef USE_CAIRO
         cairo_t *cr;
         cairo_pattern_t *pat;
-#else
-        GdkColor color;
-        GdkGC *marker;
-#endif
 
-#ifdef USE_CAIRO
         cr = gdk_cairo_create(priv->pixmap);
 
         // draw transparent area
@@ -640,47 +620,15 @@ osm_gps_map_draw_gps_point (OsmGpsMap *map)
             cairo_arc (cr, x, y, r, 0, 2 * M_PI);
             cairo_stroke(cr);
         }
-            
+
         cairo_destroy(cr);
         gtk_widget_queue_draw_area (GTK_WIDGET(map),
                                     x-mr,
                                     y-mr,
                                     mr*2,
                                     mr*2);
-#else
-        marker = gdk_gc_new(priv->pixmap);
-        color.red = 5000;
-        color.green = 5000;
-        color.blue = 55000;
-        gdk_gc_set_rgb_fg_color(marker, &color);
-        gdk_gc_set_line_attributes(marker, lw, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
-
-        if (r2 > 0) {
-            gdk_draw_arc (priv->pixmap,
-                          marker,
-                          FALSE,            //filled
-                          x-r2, y-r2,       // x,y
-                          r2*2,r2*2,        // width, height
-                          0, 360*64);       // start-end angle 64th, from 3h, anti clockwise
-        }
-        if (r > 0) {
-            gdk_draw_arc (priv->pixmap,
-                          marker,
-                          TRUE,         //filled
-                          x-r, y-r,     // x,y
-                          r*2,r*2,      // width, height
-                          0, 360*64);   // start-end angle 64th, from 3h, anti clockwise
-        }
-
-        g_object_unref(marker);
-        gtk_widget_queue_draw_area (GTK_WIDGET(map),
-                                    x-(mr+lw),
-                                    y-(mr+lw),
-                                    (mr*2)+lw+lw,
-                                    (mr*2)+lw+lw);
-#endif
     }
-} 
+}
 
 static void
 osm_gps_map_blit_tile(OsmGpsMap *map, GdkPixbuf *pixbuf, int offset_x, int offset_y)
@@ -1133,28 +1081,13 @@ osm_gps_map_print_track (OsmGpsMap *map, GSList *trackpoint_list)
     int min_x = 0,min_y = 0,max_x = 0,max_y = 0;
     int lw = priv->ui_gps_track_width;
     int map_x0, map_y0;
-#ifdef USE_CAIRO
     cairo_t *cr;
-#else
-    int last_x = 0, last_y = 0;
-    GdkColor color;
-    GdkGC *gc;
-#endif
 
-#ifdef USE_CAIRO
     cr = gdk_cairo_create(priv->pixmap);
     cairo_set_line_width (cr, lw);
     cairo_set_source_rgba (cr, 60000.0/65535.0, 0.0, 0.0, 0.6);
     cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
     cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
-#else
-    gc = gdk_gc_new(priv->pixmap);
-    color.green = 0;
-    color.blue = 0;
-    color.red = 60000;
-    gdk_gc_set_rgb_fg_color(gc, &color);
-    gdk_gc_set_line_attributes(gc, lw, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
-#endif
 
     map_x0 = priv->map_x - EXTRA_BORDER;
     map_y0 = priv->map_y - EXTRA_BORDER;
@@ -1167,21 +1100,10 @@ osm_gps_map_print_track (OsmGpsMap *map, GSList *trackpoint_list)
 
         // first time through loop
         if (list == trackpoint_list) {
-#ifdef USE_CAIRO
             cairo_move_to(cr, x, y);
-#else
-            last_x = x;
-            last_y = y;
-#endif
         }
 
-#ifdef USE_CAIRO
         cairo_line_to(cr, x, y);
-#else
-        gdk_draw_line (priv->pixmap, gc, x, y, last_x, last_y);
-        last_x = x;
-        last_y = y;
-#endif
 
         max_x = MAX(x,max_x);
         min_x = MIN(x,min_x);
@@ -1196,12 +1118,8 @@ osm_gps_map_print_track (OsmGpsMap *map, GSList *trackpoint_list)
                                 max_x + (lw * 2),
                                 max_y + (lw * 2));
 
-#ifdef USE_CAIRO
     cairo_stroke(cr);
     cairo_destroy(cr);
-#else
-    g_object_unref(gc);
-#endif
 }
 
 /* Prints the gps trip history, and any other tracks */
@@ -1319,8 +1237,10 @@ osm_gps_map_map_redraw_idle (OsmGpsMap *map)
 }
 
 static void
-center_coord_update(GtkWidget *widget) {
-    OsmGpsMapPrivate *priv = OSM_GPS_MAP_PRIVATE(widget);
+center_coord_update(OsmGpsMap *map) {
+
+    GtkWidget *widget = GTK_WIDGET(map);
+    OsmGpsMapPrivate *priv = OSM_GPS_MAP_PRIVATE(map);
 
     // pixel_x,y, offsets
     gint pixel_x = priv->map_x + widget->allocation.width/2;
@@ -1328,91 +1248,87 @@ center_coord_update(GtkWidget *widget) {
 
     priv->center_rlon = pixel2lon(priv->map_zoom, pixel_x);
     priv->center_rlat = pixel2lat(priv->map_zoom, pixel_y);
+
+    g_signal_emit_by_name(widget, "changed");
 }
 
-#ifdef OSM_GPS_MAP_KEYS
 static gboolean 
-on_window_key_press(GtkWidget *widget, 
-			 GdkEventKey *event, OsmGpsMapPrivate *priv) {
-  gboolean handled = FALSE;
-  int step = GTK_WIDGET(widget)->allocation.width/OSM_GPS_MAP_SCROLL_STEP;
+on_window_key_press(GtkWidget *widget, GdkEventKey *event, OsmGpsMapPrivate *priv) 
+{
+    int i;
+    int step;
+    gboolean handled;
+    OsmGpsMap *map;
 
-  // the map handles some keys on its own ...
-  switch(event->keyval) {
-#ifdef OSM_GPS_MAP_KEY_FULLSCREEN
-  case OSM_GPS_MAP_KEY_FULLSCREEN: {
-      GtkWidget *toplevel = gtk_widget_get_toplevel(GTK_WIDGET(widget));
-      if(!priv->fullscreen)
-          gtk_window_fullscreen(GTK_WINDOW(toplevel));
-      else
-          gtk_window_unfullscreen(GTK_WINDOW(toplevel));
+    //if no keybindings are set, let the app handle them...
+    if (!priv->keybindings_enabled)
+        return FALSE;
 
-      priv->fullscreen = !priv->fullscreen;
-      handled = TRUE;
-      } break;
-#endif
-  
-#ifdef OSM_GPS_MAP_KEY_ZOOMIN
-  case OSM_GPS_MAP_KEY_ZOOMIN:
-      osm_gps_map_set_zoom(OSM_GPS_MAP(widget), priv->map_zoom+1);
-      handled = TRUE;
-      break;
-#endif
+    handled = FALSE;
+    map = OSM_GPS_MAP(widget);
+    step = GTK_WIDGET(widget)->allocation.width/OSM_GPS_MAP_SCROLL_STEP;
 
-#ifdef OSM_GPS_MAP_KEY_ZOOMOUT
-  case OSM_GPS_MAP_KEY_ZOOMOUT:
-      osm_gps_map_set_zoom(OSM_GPS_MAP(widget), priv->map_zoom-1);
-      handled = TRUE;
-      break;
-#endif
+    //the map handles some keys on its own
+    for (i = 0; i < OSM_GPS_MAP_KEY_MAX; i++) {
+        //not the key we have a binding for
+        if (map->priv->keybindings[i] != event->keyval)
+            continue;
 
-#ifdef OSM_GPS_MAP_KEY_UP
-  case OSM_GPS_MAP_KEY_UP:
-      priv->map_y -= step;
-      center_coord_update(widget);
-      osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
-      handled = TRUE;
-      break;
-#endif
+        switch(i) {
+            case OSM_GPS_MAP_KEY_FULLSCREEN: {
+                GtkWidget *toplevel = gtk_widget_get_toplevel(GTK_WIDGET(widget));
+                if(!priv->fullscreen)
+                    gtk_window_fullscreen(GTK_WINDOW(toplevel));
+                else
+                    gtk_window_unfullscreen(GTK_WINDOW(toplevel));
 
-#ifdef OSM_GPS_MAP_KEY_DOWN
-  case OSM_GPS_MAP_KEY_DOWN:
-      priv->map_y += step;
-      center_coord_update(widget);
-      osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
-      handled = TRUE;
-      break;
-#endif
+                priv->fullscreen = !priv->fullscreen;
+                handled = TRUE;
+                } break;
+            case OSM_GPS_MAP_KEY_ZOOMIN:
+                osm_gps_map_set_zoom(map, priv->map_zoom+1);
+                handled = TRUE;
+                break;
+            case OSM_GPS_MAP_KEY_ZOOMOUT:
+                osm_gps_map_set_zoom(map, priv->map_zoom-1);
+                handled = TRUE;
+                break;
+            case OSM_GPS_MAP_KEY_UP:
+                priv->map_y -= step;
+                center_coord_update(map);
+                osm_gps_map_map_redraw_idle(map);
+                handled = TRUE;
+                break;
+            case OSM_GPS_MAP_KEY_DOWN:
+                priv->map_y += step;
+                center_coord_update(map);
+                osm_gps_map_map_redraw_idle(map);
+                handled = TRUE;
+                break;
+              case OSM_GPS_MAP_KEY_LEFT:
+                priv->map_x -= step;
+                center_coord_update(map);
+                osm_gps_map_map_redraw_idle(map);
+                handled = TRUE;
+                break;
+            case OSM_GPS_MAP_KEY_RIGHT:
+                priv->map_x += step;
+                center_coord_update(map);
+                osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
+                handled = TRUE;
+                break;
+            default:
+                break;
+        }
+    }
 
-#ifdef OSM_GPS_MAP_KEY_LEFT
-  case OSM_GPS_MAP_KEY_LEFT:
-      priv->map_x -= step;
-      center_coord_update(widget);
-      osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
-      handled = TRUE;
-      break;
-#endif
-
-#ifdef OSM_GPS_MAP_KEY_RIGHT
-  case OSM_GPS_MAP_KEY_RIGHT:
-      priv->map_x += step;
-      center_coord_update(widget);
-      osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
-      handled = TRUE;
-      break;
-#endif
-
-  default:
-      break;
-  }
-
-  return handled;
+    return handled;
 }
-#endif
 
 static void
 osm_gps_map_init (OsmGpsMap *object)
 {
+    int i;
     OsmGpsMapPrivate *priv;
 
     priv = G_TYPE_INSTANCE_GET_PRIVATE (object, OSM_TYPE_GPS_MAP, OsmGpsMapPrivate);
@@ -1428,11 +1344,6 @@ osm_gps_map_init (OsmGpsMap *object)
 #ifdef ENABLE_OSD
     priv->osd = NULL;
 #endif
-
-#ifdef OSM_GPS_MAP_BUTTON_FULLSCREEN
-    priv->fullscreen = FALSE;
-#endif
-
     priv->tracks = NULL;
     priv->images = NULL;
 
@@ -1447,6 +1358,10 @@ osm_gps_map_init (OsmGpsMap *object)
 
     priv->map_source = -1;
 
+    priv->keybindings_enabled = FALSE;
+    for (i = 0; i < OSM_GPS_MAP_KEY_MAX; i++)
+        priv->keybindings[i] = 0;
+
 #ifndef LIBSOUP22
     //Change naumber of concurrent connections option?
     priv->soup_session =
@@ -1477,17 +1392,36 @@ osm_gps_map_init (OsmGpsMap *object)
 
     g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_MASK, my_log_handler, NULL);
 
-#ifdef OSM_GPS_MAP_KEYS
+    //Setup signal handlers
     g_signal_connect(G_OBJECT(object), "key_press_event",
-                     G_CALLBACK(on_window_key_press), priv);
-#endif
+                            G_CALLBACK(on_window_key_press), priv);
 }
 
+static char*
+osm_gps_map_get_cache_dir(OsmGpsMapPrivate *priv)
+{
+    if (priv->tile_base_dir)
+        return g_strdup(priv->tile_base_dir);
+    return osm_gps_map_get_default_cache_directory();
+}
+
+/* strcmp0 was introduced with glib 2.16 */
+#if ! GLIB_CHECK_VERSION (2, 16, 0)
+int g_strcmp0(const char *str1, const char *str2)
+{
+    if( str1 == NULL && str2 == NULL ) return 0;
+    if( str1 == NULL ) return -1;
+    if( str2 == NULL ) return 1;
+    return strcmp(str1, str2);
+}
+#endif
+
 static void
-osm_gps_map_setup(OsmGpsMapPrivate *priv) {
+osm_gps_map_setup(OsmGpsMapPrivate *priv)
+{
     const char *uri;
 
-    //user can specify a map source ID, or a repo URI as the map source
+   //user can specify a map source ID, or a repo URI as the map source
     uri = osm_gps_map_source_get_repo_uri(OSM_GPS_MAP_SOURCE_NULL);
     if ( (priv->map_source == 0) || (strcmp(priv->repo_uri, uri) == 0) ) {
         g_debug("Using null source");
@@ -1511,14 +1445,30 @@ osm_gps_map_setup(OsmGpsMapPrivate *priv) {
         }
     }
 
-    const char *fname = osm_gps_map_source_get_friendly_name(priv->map_source);
-    if(!fname) fname = "_unknown_";
+    if (priv->tile_dir == NULL)
+        priv->tile_dir = g_strdup(OSM_GPS_MAP_CACHE_DISABLED);
 
-    if (priv->tile_dir) {
-        //the new cachedir is the given cache dir + the friendly name of the repo_uri
-        priv->cache_dir = g_strdup_printf("%s%c%s", priv->tile_dir, G_DIR_SEPARATOR, fname);
-        g_debug("Adjusting cache dir %s -> %s", priv->tile_dir, priv->cache_dir);
+    if ( g_strcmp0(priv->tile_dir, OSM_GPS_MAP_CACHE_DISABLED) == 0 ) {
+        priv->cache_dir = NULL;
+    } else if ( g_strcmp0(priv->tile_dir, OSM_GPS_MAP_CACHE_AUTO) == 0 ) {
+        char *base = osm_gps_map_get_cache_dir(priv);
+#if GLIB_CHECK_VERSION (2, 16, 0)
+        char *md5 = g_compute_checksum_for_string (G_CHECKSUM_MD5, priv->repo_uri, -1);
+#else
+        char *md5 = g_strdup(osm_gps_map_source_get_friendly_name(priv->map_source));
+#endif
+        priv->cache_dir = g_strdup_printf("%s%c%s", base, G_DIR_SEPARATOR, md5);
+        g_free(base);
+        g_free(md5);
+    } else if ( g_strcmp0(priv->tile_dir, OSM_GPS_MAP_CACHE_FRIENDLY) == 0 ) {
+        char *base = osm_gps_map_get_cache_dir(priv);
+        const char *fname = osm_gps_map_source_get_friendly_name(priv->map_source);
+        priv->cache_dir = g_strdup_printf("%s%c%s", base, G_DIR_SEPARATOR, fname);
+        g_free(base);
+    } else {
+        priv->cache_dir = g_strdup(priv->tile_dir);
     }
+    g_debug("Cache dir: %s", priv->cache_dir);
 }
 
 static GObject *
@@ -1573,8 +1523,10 @@ osm_gps_map_dispose (GObject *object)
     g_free(priv->gps);
 
 #ifdef ENABLE_OSD
-    if(priv->osd)
+    if(priv->osd) {
         priv->osd->free(priv->osd);
+        priv->osd = NULL;
+    }
 
 #ifdef OSD_DOUBLE_BUFFER
     if(priv->dbuf_pixmap)
@@ -1591,10 +1543,10 @@ osm_gps_map_finalize (GObject *object)
     OsmGpsMap *map = OSM_GPS_MAP(object);
     OsmGpsMapPrivate *priv = map->priv;
 
-    if(priv->tile_dir)
+    if (priv->tile_dir)
         g_free(priv->tile_dir);
 
-    if(priv->cache_dir)
+    if (priv->cache_dir)
         g_free(priv->cache_dir);
 
     g_free(priv->repo_uri);
@@ -1652,9 +1604,14 @@ osm_gps_map_set_property (GObject *object, guint prop_id, const GValue *value, G
 
             break;
         case PROP_TILE_CACHE_DIR:
-            if ( g_value_get_string(value) )
-                priv->tile_dir = g_value_dup_string (value);
+            priv->tile_dir = g_value_dup_string (value);
             break;
+        case PROP_TILE_CACHE_BASE_DIR:
+            priv->tile_base_dir = g_value_dup_string (value);
+            break;
+        case PROP_TILE_CACHE_DIR_IS_FULL_PATH:
+             g_warning("GObject property tile-cache-is-full-path depreciated");
+             break;
         case PROP_ZOOM:
             priv->map_zoom = g_value_get_int (value);
             break;
@@ -1666,11 +1623,11 @@ osm_gps_map_set_property (GObject *object, guint prop_id, const GValue *value, G
             break;
         case PROP_MAP_X:
             priv->map_x = g_value_get_int (value);
-            center_coord_update(GTK_WIDGET(object));
+            center_coord_update(map);
             break;
         case PROP_MAP_Y:
             priv->map_y = g_value_get_int (value);
-            center_coord_update(GTK_WIDGET(object));
+            center_coord_update(map);
             break;
         case PROP_GPS_TRACK_WIDTH:
             priv->ui_gps_track_width = g_value_get_int (value);
@@ -1745,6 +1702,12 @@ osm_gps_map_get_property (GObject *object, guint prop_id, GValue *value, GParamS
         case PROP_TILE_CACHE_DIR:
             g_value_set_string(value, priv->cache_dir);
             break;
+        case PROP_TILE_CACHE_BASE_DIR:
+            g_value_set_string(value, priv->tile_base_dir);
+            break;
+        case PROP_TILE_CACHE_DIR_IS_FULL_PATH:
+            g_value_set_boolean(value, FALSE);
+            break;
         case PROP_ZOOM:
             g_value_set_int(value, priv->map_zoom);
             break;
@@ -1811,6 +1774,7 @@ osm_gps_map_scroll_event (GtkWidget *widget, GdkEventScroll  *event)
 static gboolean
 osm_gps_map_button_press (GtkWidget *widget, GdkEventButton *event)
 {
+    OsmGpsMap *map = OSM_GPS_MAP(widget);
     OsmGpsMapPrivate *priv = OSM_GPS_MAP_PRIVATE(widget);
 
 #ifdef ENABLE_OSD
@@ -1828,28 +1792,28 @@ osm_gps_map_button_press (GtkWidget *widget, GdkEventButton *event)
             switch(but) {
             case OSD_UP:
                 priv->map_y -= step;
-                center_coord_update(widget);
+                center_coord_update(map);
                 g_object_set(G_OBJECT(widget), "auto-center", FALSE, NULL);
                 osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
                 break;
 
             case OSD_DOWN:
                 priv->map_y += step;
-                center_coord_update(widget);
+                center_coord_update(map);
                 g_object_set(G_OBJECT(widget), "auto-center", FALSE, NULL);
                 osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
                 break;
 
             case OSD_LEFT:
                 priv->map_x -= step;
-                center_coord_update(widget);
+                center_coord_update(map);
                 g_object_set(G_OBJECT(widget), "auto-center", FALSE, NULL);
                 osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
                 break;
                 
             case OSD_RIGHT:
                 priv->map_x += step;
-                center_coord_update(widget);
+                center_coord_update(map);
                 g_object_set(G_OBJECT(widget), "auto-center", FALSE, NULL);
                 osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
                 break;
@@ -1887,6 +1851,7 @@ osm_gps_map_button_press (GtkWidget *widget, GdkEventButton *event)
 static gboolean
 osm_gps_map_button_release (GtkWidget *widget, GdkEventButton *event)
 {
+    OsmGpsMap *map = OSM_GPS_MAP(widget);
     OsmGpsMapPrivate *priv = OSM_GPS_MAP_PRIVATE(widget);
 
     if(!priv->button_down)
@@ -1902,9 +1867,9 @@ osm_gps_map_button_release (GtkWidget *widget, GdkEventButton *event)
         priv->map_x += (priv->drag_start_mouse_x - (int) event->x);
         priv->map_y += (priv->drag_start_mouse_y - (int) event->y);
 
-        center_coord_update(widget);
+        center_coord_update(map);
 
-        osm_gps_map_map_redraw_idle(OSM_GPS_MAP(widget));
+        osm_gps_map_map_redraw_idle(map);
     }
 #ifdef ENABLE_OSD
     /* pressed inside OSD control? */
@@ -2034,6 +1999,8 @@ osm_gps_map_configure (GtkWidget *widget, GdkEventConfigure *event)
 
     osm_gps_map_map_redraw(OSM_GPS_MAP(widget));
 
+    g_signal_emit_by_name(widget, "changed");
+
     return FALSE;
 }
 
@@ -2211,14 +2178,30 @@ osm_gps_map_class_init (OsmGpsMapClass *klass)
                                      g_param_spec_string ("tile-cache",
                                                           "tile cache",
                                                           "osm local tile cache dir",
+                                                          OSM_GPS_MAP_CACHE_AUTO,
+                                                          G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_object_class_install_property (object_class,
+                                     PROP_TILE_CACHE_BASE_DIR,
+                                     g_param_spec_string ("tile-cache-base",
+                                                          "tile cache-base",
+                                                          "base directory to which friendly and auto paths are appended",
                                                           NULL,
                                                           G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 
+     g_object_class_install_property (object_class,
+                                      PROP_TILE_CACHE_DIR_IS_FULL_PATH,
+                                      g_param_spec_boolean ("tile-cache-is-full-path",
+                                                            "tile cache is full path",
+                                                            "DEPRECIATED",
+                                                            FALSE,
+                                                            G_PARAM_READABLE | G_PARAM_WRITABLE));
+
     g_object_class_install_property (object_class,
                                      PROP_ZOOM,
                                      g_param_spec_int ("zoom",
                                                        "zoom",
-                                                       "zoom level",
+                                                       "initial zoom level",
                                                        MIN_ZOOM, /* minimum property value */
                                                        MAX_ZOOM, /* maximum property value */
                                                        3,
@@ -2311,7 +2294,7 @@ osm_gps_map_class_init (OsmGpsMapClass *klass)
                                                        "radius of the gps point inner circle",
                                                        0,           /* minimum property value */
                                                        G_MAXINT,    /* maximum property value */
-                                                       10,
+                                                       5,
                                                        G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
 
     g_object_class_install_property (object_class,
@@ -2341,6 +2324,10 @@ osm_gps_map_class_init (OsmGpsMapClass *klass)
                                                           "map source tile repository image format (jpg, png)",
                                                           OSM_IMAGE_FORMAT,
                                                           G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_signal_new ("changed", OSM_TYPE_GPS_MAP,
+                  G_SIGNAL_RUN_FIRST, 0, NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 }
 
 const char* 
@@ -2354,6 +2341,8 @@ osm_gps_map_source_get_friendly_name(OsmGpsMapSource_t source)
             return "OpenStreetMap I";
         case OSM_GPS_MAP_SOURCE_OPENSTREETMAP_RENDERER:
             return "OpenStreetMap II";
+        case OSM_GPS_MAP_SOURCE_OPENAERIALMAP:
+            return "OpenAerialMap";
         case OSM_GPS_MAP_SOURCE_OPENCYCLEMAP:
             return "OpenCycleMap";
         case OSM_GPS_MAP_SOURCE_OSM_PUBLIC_TRANSPORT:
@@ -2380,6 +2369,7 @@ osm_gps_map_source_get_friendly_name(OsmGpsMapSource_t source)
             return "Yahoo Satellite";
         case OSM_GPS_MAP_SOURCE_YAHOO_HYBRID:
             return "Yahoo Hybrid";
+        case OSM_GPS_MAP_SOURCE_LAST:
         default:
             return NULL;
     }
@@ -2399,6 +2389,10 @@ osm_gps_map_source_get_repo_uri(OsmGpsMapSource_t source)
             return "none://";
         case OSM_GPS_MAP_SOURCE_OPENSTREETMAP:
             return OSM_REPO_URI;
+        case OSM_GPS_MAP_SOURCE_OPENAERIALMAP:
+            /* OpenAerialMap is down, offline till furthur notice
+               http://openaerialmap.org/pipermail/talk_openaerialmap.org/2008-December/000055.html */
+            return NULL;
         case OSM_GPS_MAP_SOURCE_OPENSTREETMAP_RENDERER:
             return "http://tah.openstreetmap.org/Tiles/tile/#Z/#X/#Y.png";
         case OSM_GPS_MAP_SOURCE_OPENCYCLEMAP:
@@ -2411,10 +2405,12 @@ osm_gps_map_source_get_repo_uri(OsmGpsMapSource_t source)
             return "http://maps-for-free.com/layer/relief/z#Z/row#Y/#Z_#X-#Y.jpg";
         case OSM_GPS_MAP_SOURCE_GOOGLE_STREET:
             return "http://mt#R.google.com/vt/v=w2.97&x=#X&y=#Y&z=#Z";
-        case OSM_GPS_MAP_SOURCE_GOOGLE_SATELLITE:
-            return "http://khm#R.google.com/kh?n=404&v=3&t=#Q";
         case OSM_GPS_MAP_SOURCE_GOOGLE_HYBRID:
-            return NULL; /* No longer working  "http://mt#R.google.com/mt?n=404&v=w2t.99&x=#X&y=#Y&zoom=#S" */
+            /* No longer working
+               "http://mt#R.google.com/mt?n=404&v=w2t.99&x=#X&y=#Y&zoom=#S" */
+            return NULL;
+        case OSM_GPS_MAP_SOURCE_GOOGLE_SATELLITE:
+            return "http://khm#R.google.com/kh/v=51&x=#X&y=#Y&z=#Z";
         case OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_STREET:
             return "http://a#R.ortho.tiles.virtualearth.net/tiles/r#W.jpeg?g=50";
         case OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_SATELLITE:
@@ -2431,6 +2427,7 @@ osm_gps_map_source_get_repo_uri(OsmGpsMapSource_t source)
              *  z = zoom - (MAX_ZOOM - 17));
              */
             return NULL;
+        case OSM_GPS_MAP_SOURCE_LAST:
         default:
             return NULL;
     }
@@ -2448,9 +2445,8 @@ osm_gps_map_source_get_image_format(OsmGpsMapSource_t source)
         case OSM_GPS_MAP_SOURCE_OSM_PUBLIC_TRANSPORT:
         case OSM_GPS_MAP_SOURCE_OSMC_TRAILS:
             return "png";
-        case OSM_GPS_MAP_SOURCE_MAPS_FOR_FREE:
+        case OSM_GPS_MAP_SOURCE_OPENAERIALMAP:
         case OSM_GPS_MAP_SOURCE_GOOGLE_STREET:
-        case OSM_GPS_MAP_SOURCE_GOOGLE_SATELLITE:
         case OSM_GPS_MAP_SOURCE_GOOGLE_HYBRID:
         case OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_STREET:
         case OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_SATELLITE:
@@ -2458,7 +2454,10 @@ osm_gps_map_source_get_image_format(OsmGpsMapSource_t source)
         case OSM_GPS_MAP_SOURCE_YAHOO_STREET:
         case OSM_GPS_MAP_SOURCE_YAHOO_SATELLITE:
         case OSM_GPS_MAP_SOURCE_YAHOO_HYBRID:
+        case OSM_GPS_MAP_SOURCE_MAPS_FOR_FREE:
+        case OSM_GPS_MAP_SOURCE_GOOGLE_SATELLITE:
             return "jpg";
+        case OSM_GPS_MAP_SOURCE_LAST:
         default:
             return "bin";
     }
@@ -2483,6 +2482,7 @@ osm_gps_map_source_get_max_zoom(OsmGpsMapSource_t source)
         case OSM_GPS_MAP_SOURCE_OSM_PUBLIC_TRANSPORT:
             return OSM_MAX_ZOOM;
         case OSM_GPS_MAP_SOURCE_OPENSTREETMAP_RENDERER:
+        case OSM_GPS_MAP_SOURCE_OPENAERIALMAP:
         case OSM_GPS_MAP_SOURCE_GOOGLE_STREET:
         case OSM_GPS_MAP_SOURCE_GOOGLE_HYBRID:
         case OSM_GPS_MAP_SOURCE_VIRTUAL_EARTH_STREET:
@@ -2498,12 +2498,19 @@ osm_gps_map_source_get_max_zoom(OsmGpsMapSource_t source)
             return 11;
         case OSM_GPS_MAP_SOURCE_GOOGLE_SATELLITE:
             return 18;
+        case OSM_GPS_MAP_SOURCE_LAST:
         default:
             return 17;
     }
     return 17;
 }
 
+gboolean
+osm_gps_map_source_is_valid(OsmGpsMapSource_t source)
+{
+    return osm_gps_map_source_get_repo_uri(source) != NULL;
+}
+
 void
 osm_gps_map_download_maps (OsmGpsMap *map, coord_t *pt1, coord_t *pt2, int zoom_start, int zoom_end)
 {
@@ -2595,6 +2602,8 @@ osm_gps_map_set_center (OsmGpsMap *map, float latitude, float longitude)
     priv->map_y = pixel_y - GTK_WIDGET(map)->allocation.height/2;
 
     osm_gps_map_map_redraw_idle(map);
+
+    g_signal_emit_by_name(map, "changed");
 }
 
 int 
@@ -2634,6 +2643,8 @@ osm_gps_map_set_zoom (OsmGpsMap *map, int zoom)
 #endif
 
         osm_gps_map_map_redraw_idle(map);
+
+        g_signal_emit_by_name(map, "changed");
     }
     return priv->map_zoom;
 }
@@ -2685,7 +2696,7 @@ osm_gps_map_clear_tracks (OsmGpsMap *map)
 }
 
 void
-osm_gps_map_add_image (OsmGpsMap *map, float latitude, float longitude, GdkPixbuf *image)
+osm_gps_map_add_image_with_alignment (OsmGpsMap *map, float latitude, float longitude, GdkPixbuf *image, float xalign, float yalign)
 {
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
@@ -2700,6 +2711,10 @@ osm_gps_map_add_image (OsmGpsMap *map, float latitude, float longitude, GdkPixbu
         im->pt.rlat = deg2rad(latitude);
         im->pt.rlon = deg2rad(longitude);
 
+        //handle alignment
+        im->xoffset = xalign * im->w;
+        im->yoffset = yalign * im->h;
+
         g_object_ref(image);
         im->image = image;
 
@@ -2709,6 +2724,12 @@ osm_gps_map_add_image (OsmGpsMap *map, float latitude, float longitude, GdkPixbu
     }
 }
 
+void
+osm_gps_map_add_image (OsmGpsMap *map, float latitude, float longitude, GdkPixbuf *image)
+{
+    osm_gps_map_add_image_with_alignment (map, latitude, longitude, image, 0.5, 0.5);
+}
+
 gboolean
 osm_gps_map_remove_image (OsmGpsMap *map, GdkPixbuf *image)
 {
@@ -2783,7 +2804,7 @@ osm_gps_map_draw_gps (OsmGpsMap *map, float latitude, float longitude, float hea
 
             priv->map_x = pixel_x - GTK_WIDGET(map)->allocation.width/2;
             priv->map_y = pixel_y - GTK_WIDGET(map)->allocation.height/2;
-            center_coord_update(GTK_WIDGET(map));
+            center_coord_update(map);
         }
     }
 
@@ -2842,10 +2863,10 @@ osm_gps_map_geographic_to_screen (OsmGpsMap *map,
     priv = map->priv;
 
     if (pixel_x)
-        *pixel_x = lon2pixel(priv->map_zoom, deg2rad(longitude)) - 
+        *pixel_x = lon2pixel(priv->map_zoom, deg2rad(longitude)) -
             priv->map_x + priv->drag_mouse_dx;
     if (pixel_y)
-        *pixel_y = lat2pixel(priv->map_zoom, deg2rad(latitude)) - 
+        *pixel_y = lat2pixel(priv->map_zoom, deg2rad(latitude)) -
             priv->map_y + priv->drag_mouse_dy;
 }
 
@@ -2859,7 +2880,7 @@ osm_gps_map_scroll (OsmGpsMap *map, gint dx, gint dy)
 
     priv->map_x += dx;
     priv->map_y += dy;
-    center_coord_update(GTK_WIDGET(map));
+    center_coord_update(map);
 
 #ifdef ENABLE_OSD
     /* OSD may contain a coordinate, so we may have to re-render it */
@@ -2881,6 +2902,23 @@ osm_gps_map_get_scale(OsmGpsMap *map)
     return osm_gps_map_get_scale_at_point(priv->map_zoom, priv->center_rlat, priv->center_rlon);
 }
 
+char * osm_gps_map_get_default_cache_directory(void)
+{
+    return g_build_filename(
+                        g_get_user_cache_dir(),
+                        "osmgpsmap",
+                        NULL);
+}
+
+void osm_gps_map_set_keyboard_shortcut(OsmGpsMap *map, OsmGpsMapKey_t key, guint keyval)
+{
+    g_return_if_fail (OSM_IS_GPS_MAP (map));
+    g_return_if_fail(key < OSM_GPS_MAP_KEY_MAX);
+
+    map->priv->keybindings[key] = keyval;
+    map->priv->keybindings_enabled = TRUE;
+}
+
 #ifdef ENABLE_OSD
 
 void
